package model;

import data_access.DataRepository;
import org.junit.Before;
import org.junit.Test;
import testsuites.TestBuilder;

import java.util.Optional;

import static org.junit.Assert.*;
import static org.junit.Assert.assertNull;

public class ArticleTest {


    public final static String idTasse = "SKU-458362";
    public final static String idTest = "SKU-458000";
    private DataRepository<Article,String> articleData = null;

    @Before
    public void setUp() throws Exception {
        //System.out.println( this.getClass().getSimpleName() + ".setUp() called." );
        this.articleData = TestBuilder.getInstance().dataAccess().getArticleData();
    }

    @Test
    public void testConstructor(){

        Article articleToaster = new Article("SKU-868682","Toaster",2499, 1200);
        assertNotNull(articleToaster.getId());
        assertEquals(articleToaster.getId(), "SKU-868682");
        assertEquals(articleToaster.getDescription(), "Toaster");
        assertEquals(articleToaster.getUnitPrice(), 2499);
        assertEquals(articleToaster.getUnitsInStore(), 1200);

    }

    @Test
    public void testNullConstructor() {
        Article toast = new Article( null, null, 0, 0);
        assertEquals( toast.getId(), null);
        assertEquals( toast.getDescription(), "" );
        assertEquals( toast.getUnitPrice(), 0 );
        assertEquals( toast.getUnitsInStore(),0 );
    }

    @Test
    public void testNegConstructor(){
        Article toast = new Article( "", "", -10, -100);
        assertEquals(toast.getId(), null);
        assertEquals( toast.getDescription(), "" );
        assertEquals( toast.getUnitPrice(), 0 );
        assertEquals( toast.getUnitsInStore(),0 );
    }

    @Test
    public void testArticleData(){

        assertNotNull( articleData);
        Optional<Article> optArt = articleData.findById(idTasse);
        assertTrue(optArt.isPresent());
        Article tasse = optArt.get();
        assertNotNull( tasse );
        assertEquals( tasse.getDescription(), "Tasse" );
        assertEquals( tasse.getUnitPrice(), 299 );
        tasse.setUnitPrice(-5);
        assertEquals(tasse.getUnitPrice(),0);
        tasse.setDescription(null);
        assertEquals(tasse.getDescription(),"");
        tasse.setUnitPrice(299);
        tasse.setDescription("Tasse");
        Optional<Article> falsePositive = articleData.findById( idTest );
        assertFalse( falsePositive.isPresent() );


    }

}

